@@ -16,6 +16,8 @@
 
 package dalvik.system;
 
+import dalvik.annotation.compat.UnsupportedAppUsage;
+import dalvik.annotation.optimization.FastNative;
 import java.io.FileDescriptor;
 import java.io.IOException;
 import java.util.HashMap;
@@ -30,11 +32,13 @@ import java.util.Map;
  *
  * @hide
  */
+@libcore.api.CorePlatformApi
 public final class VMDebug {
     /**
      * flag for startMethodTracing(), which adds the results from
      * startAllocCounting to the trace key file.
      */
+    @libcore.api.CorePlatformApi
     public static final int TRACE_COUNT_ALLOCS = 1;
 
     /* constants for getAllocCount */
@@ -50,18 +54,25 @@ public final class VMDebug {
     private static final int KIND_EXT_FREED_OBJECTS     = 1<<14;
     private static final int KIND_EXT_FREED_BYTES       = 1<<15;
 
+    @libcore.api.CorePlatformApi
     public static final int KIND_GLOBAL_ALLOCATED_OBJECTS =
         KIND_ALLOCATED_OBJECTS;
+    @libcore.api.CorePlatformApi
     public static final int KIND_GLOBAL_ALLOCATED_BYTES =
         KIND_ALLOCATED_BYTES;
+    @libcore.api.CorePlatformApi
     public static final int KIND_GLOBAL_FREED_OBJECTS =
         KIND_FREED_OBJECTS;
+    @libcore.api.CorePlatformApi
     public static final int KIND_GLOBAL_FREED_BYTES =
         KIND_FREED_BYTES;
+    @libcore.api.CorePlatformApi
     public static final int KIND_GLOBAL_GC_INVOCATIONS =
         KIND_GC_INVOCATIONS;
+    @libcore.api.CorePlatformApi
     public static final int KIND_GLOBAL_CLASS_INIT_COUNT =
         KIND_CLASS_INIT_COUNT;
+    @libcore.api.CorePlatformApi
     public static final int KIND_GLOBAL_CLASS_INIT_TIME =
         KIND_CLASS_INIT_TIME;
     public static final int KIND_GLOBAL_EXT_ALLOCATED_OBJECTS =
@@ -73,14 +84,17 @@ public final class VMDebug {
     public static final int KIND_GLOBAL_EXT_FREED_BYTES =
         KIND_EXT_FREED_BYTES;
 
+    @libcore.api.CorePlatformApi
     public static final int KIND_THREAD_ALLOCATED_OBJECTS =
         KIND_ALLOCATED_OBJECTS << 16;
+    @libcore.api.CorePlatformApi
     public static final int KIND_THREAD_ALLOCATED_BYTES =
         KIND_ALLOCATED_BYTES << 16;
     public static final int KIND_THREAD_FREED_OBJECTS =
         KIND_FREED_OBJECTS << 16;
     public static final int KIND_THREAD_FREED_BYTES =
         KIND_FREED_BYTES << 16;
+    @libcore.api.CorePlatformApi
     public static final int KIND_THREAD_GC_INVOCATIONS =
         KIND_GC_INVOCATIONS << 16;
     public static final int KIND_THREAD_CLASS_INIT_COUNT =
@@ -96,6 +110,7 @@ public final class VMDebug {
     public static final int KIND_THREAD_EXT_FREED_BYTES =
         KIND_EXT_FREED_BYTES << 16;
 
+    @libcore.api.CorePlatformApi
     public static final int KIND_ALL_COUNTS = 0xffffffff;
 
     /* all methods are static */
@@ -106,6 +121,8 @@ public final class VMDebug {
      *
      * @return the time in milliseconds, or -1 if the debugger is not connected
      */
+    @libcore.api.CorePlatformApi
+    @FastNative
     public static native long lastDebuggerActivity();
 
     /**
@@ -114,6 +131,8 @@ public final class VMDebug {
      *
      * @return true if debugging is enabled
      */
+    @libcore.api.CorePlatformApi
+    @FastNative
     public static native boolean isDebuggingEnabled();
 
     /**
@@ -121,6 +140,9 @@ public final class VMDebug {
      *
      * @return true if (and only if) a debugger is connected
      */
+    @UnsupportedAppUsage
+    @libcore.api.CorePlatformApi
+    @FastNative
     public static native boolean isDebuggerConnected();
 
     /**
@@ -128,6 +150,7 @@ public final class VMDebug {
      * used by DDMS to determine what sorts of operations the VM can
      * perform.
      */
+    @libcore.api.CorePlatformApi
     public static native String[] getVmFeatureList();
 
     /**
@@ -163,6 +186,7 @@ public final class VMDebug {
      * @param intervalUs the time between samples in microseconds when
      * sampling is enabled.
      */
+    @libcore.api.CorePlatformApi
     public static void startMethodTracing(String traceFileName, int bufferSize, int flags, boolean samplingEnabled, int intervalUs) {
         startMethodTracingFilename(traceFileName, checkBufferSize(bufferSize), flags, samplingEnabled, intervalUs);
     }
@@ -172,11 +196,27 @@ public final class VMDebug {
      * FileDescriptor in which the trace is written.  The file name is also
      * supplied simply for logging.  Makes a dup of the file descriptor.
      */
-    public static void startMethodTracing(String traceFileName, FileDescriptor fd, int bufferSize, int flags, boolean samplingEnabled, int intervalUs) {
+    public static void startMethodTracing(String traceFileName, FileDescriptor fd, int bufferSize,
+                                          int flags, boolean samplingEnabled, int intervalUs) {
+        startMethodTracing(traceFileName, fd, bufferSize, flags, samplingEnabled, intervalUs,
+                           false);
+    }
+
+    /**
+     * Like startMethodTracing(String, int, int), but taking an already-opened
+     * FileDescriptor in which the trace is written.  The file name is also
+     * supplied simply for logging.  Makes a dup of the file descriptor.
+     * Streams tracing data to the file if streamingOutput is true.
+     */
+    @libcore.api.CorePlatformApi
+    public static void startMethodTracing(String traceFileName, FileDescriptor fd, int bufferSize,
+                                          int flags, boolean samplingEnabled, int intervalUs,
+                                          boolean streamingOutput) {
         if (fd == null) {
             throw new NullPointerException("fd == null");
         }
-        startMethodTracingFd(traceFileName, fd, checkBufferSize(bufferSize), flags, samplingEnabled, intervalUs);
+        startMethodTracingFd(traceFileName, fd.getInt$(), checkBufferSize(bufferSize), flags,
+                             samplingEnabled, intervalUs, streamingOutput);
     }
 
     /**
@@ -184,6 +224,7 @@ public final class VMDebug {
      * is called, the result is sent directly to DDMS.  (If DDMS is not
      * attached when tracing ends, the profiling data will be discarded.)
      */
+    @libcore.api.CorePlatformApi
     public static void startMethodTracingDdms(int bufferSize, int flags, boolean samplingEnabled, int intervalUs) {
         startMethodTracingDdmsImpl(checkBufferSize(bufferSize), flags, samplingEnabled, intervalUs);
     }
@@ -200,28 +241,33 @@ public final class VMDebug {
     }
 
     private static native void startMethodTracingDdmsImpl(int bufferSize, int flags, boolean samplingEnabled, int intervalUs);
-    private static native void startMethodTracingFd(String traceFileName, FileDescriptor fd, int bufferSize, int flags, boolean samplingEnabled, int intervalUs);
+    private static native void startMethodTracingFd(String traceFileName, int fd, int bufferSize,
+            int flags, boolean samplingEnabled, int intervalUs, boolean streamingOutput);
     private static native void startMethodTracingFilename(String traceFileName, int bufferSize, int flags, boolean samplingEnabled, int intervalUs);
 
     /**
      * Determine whether method tracing is currently active and what type is
      * active.
      */
+    @libcore.api.CorePlatformApi
     public static native int getMethodTracingMode();
 
     /**
      * Stops method tracing.
      */
+    @libcore.api.CorePlatformApi
     public static native void stopMethodTracing();
 
     /**
      * Starts sending Dalvik method trace info to the emulator.
      */
+    @libcore.api.CorePlatformApi
     public static native void startEmulatorTracing();
 
     /**
      * Stops sending Dalvik method trace info to the emulator.
      */
+    @libcore.api.CorePlatformApi
     public static native void stopEmulatorTracing();
 
     /**
@@ -236,15 +282,21 @@ public final class VMDebug {
      * @return the CPU usage. A value of -1 means the system does not support
      *         this feature.
      */
+    @libcore.api.CorePlatformApi
+    @FastNative
     public static native long threadCpuTimeNanos();
 
     /**
      * Count the number and aggregate size of memory allocations between
      * two points.
      */
+    @libcore.api.CorePlatformApi
     public static native void startAllocCounting();
+    @libcore.api.CorePlatformApi
     public static native void stopAllocCounting();
+    @libcore.api.CorePlatformApi
     public static native int getAllocCount(int kind);
+    @libcore.api.CorePlatformApi
     public static native void resetAllocCount(int kinds);
 
     /**
@@ -276,6 +328,8 @@ public final class VMDebug {
     /**
      * Dumps a list of loaded class to the log file.
      */
+    @libcore.api.CorePlatformApi
+    @FastNative
     public static native void printLoadedClasses(int flags);
 
     /**
@@ -283,6 +337,8 @@ public final class VMDebug {
      *
      * @return the number of loaded classes
      */
+    @libcore.api.CorePlatformApi
+    @FastNative
     public static native int getLoadedClassCount();
 
     /**
@@ -295,6 +351,7 @@ public final class VMDebug {
      *         HPROF support.
      * @throws IOException if an error occurs while opening or writing files.
      */
+    @libcore.api.CorePlatformApi
     public static void dumpHprofData(String filename) throws IOException {
         if (filename == null) {
             throw new NullPointerException("filename == null");
@@ -308,6 +365,7 @@ public final class VMDebug {
      * @throws UnsupportedOperationException if the VM was built without
      *         HPROF support.
      */
+    @libcore.api.CorePlatformApi
     public static native void dumpHprofDataDdms();
 
     /**
@@ -318,18 +376,27 @@ public final class VMDebug {
      * @param fd Descriptor of open file that will receive the output.
      *        If this is null, the fileName is used instead.
      */
-    public static native void dumpHprofData(String fileName, FileDescriptor fd)
+    @libcore.api.CorePlatformApi
+    public static void dumpHprofData(String fileName, FileDescriptor fd)
+            throws IOException {
+       dumpHprofData(fileName, fd != null ? fd.getInt$() : -1);
+    }
+
+    private static native void dumpHprofData(String fileName, int fd)
             throws IOException;
 
     /**
      * Primes the register map cache.
      */
+    @libcore.api.CorePlatformApi
     public static native boolean cacheRegisterMap(String classAndMethodDesc);
 
     /**
      * Dumps the contents of the VM reference tables (e.g. JNI locals and
      * globals) to the log file.
      */
+    @UnsupportedAppUsage
+    @libcore.api.CorePlatformApi
     public static native void dumpReferenceTables();
 
     /**
@@ -365,12 +432,13 @@ public final class VMDebug {
      * objects to get counted.
      *
      * @param klass the class to be counted.
-     * @param assignable if false, direct instances of klass are
-     *                   counted.  If true, instances that are
-     *                   assignable to klass, as defined by
-     *                   {@link Class#isAssignableFrom} are counted.
+     * @param assignable if true, any instance whose class is assignable to
+     *                   <code>klass</code>, as defined by {@link Class#isAssignableFrom},
+     *                   is counted. If false, only instances whose class is
+     *                   equal to <code>klass</code> are counted.
      * @return the number of matching instances.
      */
+    @libcore.api.CorePlatformApi
     public static native long countInstancesOfClass(Class klass, boolean assignable);
 
     /**
@@ -379,15 +447,33 @@ public final class VMDebug {
      * objects to get counted.
      *
      * @param classes the classes to be counted.
-     * @param assignable if false, direct instances of klass are
-     *                   counted.  If true, instances that are
-     *                   assignable to klass, as defined by
-     *                   {@link Class#isAssignableFrom} are counted.
-     * @return an array containing the number of matching instances. The value for
-     * an index is the number of instances of the class at that index in number classes.
+     * @param assignable if true, any instance whose class is assignable to
+     *                   <code>classes[i]</code>, as defined by {@link Class#isAssignableFrom},
+     *                   is counted. If false, only instances whose class is
+     *                   equal to <code>classes[i]</code> are counted.
+     * @return an array containing the number of matching instances. The value
+     *         for index <code>i</code> is the number of instances of
+     *         the class <code>classes[i]</code>
      */
+    @libcore.api.CorePlatformApi
     public static native long[] countInstancesOfClasses(Class[] classes, boolean assignable);
 
+    /**
+     * Gets instances of classes on the Java heap.
+     * It is the caller's responsibility to do GC if they don't want unreachable
+     * objects to be included.
+     *
+     * @param classes the classes to get instances of.
+     * @param assignable if true, any instance whose class is assignable to
+     *                   <code>classes[i]</code>, as defined by {@link Class#isAssignableFrom},
+     *                   is included. If false, only instances whose class is
+     *                   equal to <code>classes[i]</code> are included.
+     * @return an array containing the list of matching instances. The value
+     *         for index <code>i</code> is an array containing the instances
+     *         of the class <code>classes[i]</code>
+     */
+    public static native Object[][] getInstancesOfClasses(Class[] classes, boolean assignable);
+
     /**
      * Export the heap per-space stats for dumpsys meminfo.
      *
@@ -431,6 +517,7 @@ public final class VMDebug {
      *            the name of the runtime statistic to look up.
      * @return the value of the runtime statistic.
      */
+    @libcore.api.CorePlatformApi
     public static String getRuntimeStat(String statName) {
         if (statName == null) {
             throw new NullPointerException("statName == null");
@@ -448,6 +535,7 @@ public final class VMDebug {
      *
      * @return a map of the names/values of the supported runtime statistics.
      */
+    @libcore.api.CorePlatformApi
     public static Map<String, String> getRuntimeStats() {
         HashMap<String, String> map = new HashMap<>();
         String[] values = getRuntimeStatsInternal();
@@ -461,4 +549,48 @@ public final class VMDebug {
 
     private static native String getRuntimeStatInternal(int statId);
     private static native String[] getRuntimeStatsInternal();
+
+    /**
+     * Attaches an agent to the VM.
+     *
+     * @param agent The path to the agent .so file plus optional agent arguments.
+     */
+    public static void attachAgent(String agent) throws IOException {
+        attachAgent(agent, null);
+    }
+
+    /**
+     * Attaches an agent to the VM.
+     *
+     * @param agent The path to the agent .so file plus optional agent arguments.
+     * @param classLoader The classloader to use as a loading context.
+     */
+    @libcore.api.CorePlatformApi
+    public static void attachAgent(String agent, ClassLoader classLoader) throws IOException {
+        nativeAttachAgent(agent, classLoader);
+    }
+
+    private static native void nativeAttachAgent(String agent, ClassLoader classLoader)
+            throws IOException;
+
+    /**
+     * Exempts a class from any future non-SDK API access checks.
+     * Methods declared in the class will be allowed to perform
+     * reflection/JNI against the framework completely unrestricted.
+     * Note that this does not affect uses of non-SDK APIs that the class links against.
+     * Note that this does not affect methods declared outside this class, e.g.
+     * inherited from a superclass or an implemented interface.
+     *
+     * @param klass The class whose methods should be exempted.
+     */
+    @UnsupportedAppUsage
+    public static native void allowHiddenApiReflectionFrom(Class<?> klass);
+
+    /**
+     * Sets the number of frames recorded for allocation tracking.
+     *
+     * @param stackDepth The number of frames captured for each stack trace.
+     */
+    @libcore.api.CorePlatformApi
+    public static native void setAllocTrackerStackDepth(int stackDepth);
 }
