@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2014 The Android Open Source Project
- * Copyright (c) 1995, 2006, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2013, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,16 +32,15 @@ import java.util.Objects;
 import sun.net.util.IPAddressUtil;
 
 /**
- * The abstract class <code>URLStreamHandler</code> is the common
+ * The abstract class {@code URLStreamHandler} is the common
  * superclass for all stream protocol handlers. A stream protocol
  * handler knows how to make a connection for a particular protocol
- * type, such as <code>http</code>, <code>ftp</code>, or
- * <code>gopher</code>.
+ * type, such as {@code http} or {@code https}.
  * <p>
- * In most cases, an instance of a <code>URLStreamHandler</code>
+ * In most cases, an instance of a {@code URLStreamHandler}
  * subclass is not created directly by an application. Rather, the
  * first time a protocol name is encountered when constructing a
- * <code>URL</code>, the appropriate stream protocol handler is
+ * {@code URL}, the appropriate stream protocol handler is
  * automatically loaded.
  *
  * @author  James Gosling
@@ -51,7 +50,7 @@ import sun.net.util.IPAddressUtil;
 public abstract class URLStreamHandler {
     /**
      * Opens a connection to the object referenced by the
-     * <code>URL</code> argument.
+     * {@code URL} argument.
      * This method should be overridden by a subclass.
      *
      * <p>If for the handler's protocol (such as HTTP or JAR), there
@@ -63,7 +62,7 @@ public abstract class URLStreamHandler {
      * JarURLConnection will be returned.
      *
      * @param      u   the URL that this connects to.
-     * @return     a <code>URLConnection</code> object for the <code>URL</code>.
+     * @return     a {@code URLConnection} object for the {@code URL}.
      * @exception  IOException  if an I/O error occurs while opening the
      *               connection.
      */
@@ -82,7 +81,7 @@ public abstract class URLStreamHandler {
      * @param      p   the proxy through which the connection will be made.
      *                 If direct connection is desired, Proxy.NO_PROXY
      *                 should be specified.
-     * @return     a <code>URLConnection</code> object for the <code>URL</code>.
+     * @return     a {@code URLConnection} object for the {@code URL}.
      * @exception  IOException  if an I/O error occurs while opening the
      *               connection.
      * @exception  IllegalArgumentException if either u or p is null,
@@ -96,28 +95,28 @@ public abstract class URLStreamHandler {
     }
 
     /**
-     * Parses the string representation of a <code>URL</code> into a
-     * <code>URL</code> object.
+     * Parses the string representation of a {@code URL} into a
+     * {@code URL} object.
      * <p>
      * If there is any inherited context, then it has already been
-     * copied into the <code>URL</code> argument.
+     * copied into the {@code URL} argument.
      * <p>
-     * The <code>parseURL</code> method of <code>URLStreamHandler</code>
+     * The {@code parseURL} method of {@code URLStreamHandler}
      * parses the string representation as if it were an
-     * <code>http</code> specification. Most URL protocol families have a
+     * {@code http} specification. Most URL protocol families have a
      * similar parsing. A stream protocol handler for a protocol that has
      * a different syntax must override this routine.
      *
-     * @param   u       the <code>URL</code> to receive the result of parsing
+     * @param   u       the {@code URL} to receive the result of parsing
      *                  the spec.
-     * @param   spec    the <code>String</code> representing the URL that
+     * @param   spec    the {@code String} representing the URL that
      *                  must be parsed.
      * @param   start   the character index at which to begin parsing. This is
-     *                  just past the '<code>:</code>' (if there is one) that
+     *                  just past the '{@code :}' (if there is one) that
      *                  specifies the determination of the protocol name.
      * @param   limit   the character position to stop parsing at. This is the
      *                  end of the string or the position of the
-     *                  "<code>#</code>" character, if present. All information
+     *                  "{@code #}" character, if present. All information
      *                  after the sharp sign indicates an anchor.
      */
     protected void parseURL(URL u, String spec, int start, int limit) {
@@ -135,11 +134,11 @@ public abstract class URLStreamHandler {
 
         boolean isRelPath = false;
         boolean queryOnly = false;
-        // ----- BEGIN android -----
+        // BEGIN Android-changed: App compat
         boolean querySet = false;
-        // ----- END android -----
+        // END Android-changed: App compat
 
-        // FIX: should not assume query if opaque
+// FIX: should not assume query if opaque
         // Strip off the query part
         if (start < limit) {
             int queryStart = spec.indexOf('?');
@@ -149,31 +148,44 @@ public abstract class URLStreamHandler {
                 if (limit > queryStart)
                     limit = queryStart;
                 spec = spec.substring(0, queryStart);
-                // ----- BEGIN android -----
+                // BEGIN Android-changed: App compat
                 querySet = true;
-                // ----- END android -----
+                // END Android-changed: App compat
             }
         }
 
         int i = 0;
         // Parse the authority part if any
-        // ----- BEGIN android -----
+        // BEGIN Android-changed: App compat
         // boolean isUNCName = (start <= limit - 4) &&
         //                 (spec.charAt(start) == '/') &&
         //                 (spec.charAt(start + 1) == '/') &&
         //                 (spec.charAt(start + 2) == '/') &&
         //                 (spec.charAt(start + 3) == '/');
         boolean isUNCName = false;
-        // ----- END android -----
+        // END Android-changed: App compat
         if (!isUNCName && (start <= limit - 2) && (spec.charAt(start) == '/') &&
             (spec.charAt(start + 1) == '/')) {
             start += 2;
+            // BEGIN Android-changed: Check for all hostname termination chars. http://b/110955991
+            /*
             i = spec.indexOf('/', start);
             if (i < 0 || i > limit) {
                 i = spec.indexOf('?', start);
                 if (i < 0 || i > limit)
                     i = limit;
             }
+            */
+            LOOP: for (i = start; i < limit; i++) {
+                switch (spec.charAt(i)) {
+                    case '/':  // Start of path
+                    case '\\': // Start of path - see https://url.spec.whatwg.org/#host-state
+                    case '?':  // Start of query
+                    case '#':  // Start of fragment
+                        break LOOP;
+                }
+            }
+            // END Android-changed: Check for all hostname termination chars. http://b/110955991
 
             host = authority = spec.substring(start, i);
 
@@ -227,7 +239,7 @@ public abstract class URLStreamHandler {
                     if (ind >= 0) {
                         // port can be null according to RFC2396
                         if (host.length() > (ind + 1)) {
-                            // ----- BEGIN android -----
+                            // BEGIN Android-changed: App compat
                             // port = Integer.parseInt(host.substring(ind + 1));
                             char firstPortChar = host.charAt(ind+1);
                             if (firstPortChar >= '0' && firstPortChar <= '9') {
@@ -236,7 +248,7 @@ public abstract class URLStreamHandler {
                                 throw new IllegalArgumentException("invalid port: " +
                                                                    host.substring(ind + 1));
                             }
-                            // ----- END android -----
+                            // END Android-changed: App compat
                         }
                         host = host.substring(0, ind);
                     }
@@ -249,16 +261,16 @@ public abstract class URLStreamHandler {
                                                    port);
             start = i;
 
-            // ----- BEGIN android -----
             // If the authority is defined then the path is defined by the
             // spec only; See RFC 2396 Section 5.2.4.
+            // BEGIN Android-changed: App compat
             // if (authority != null && authority.length() > 0)
             //   path = "";
             path = null;
             if (!querySet) {
                 query = null;
             }
-            // ----- END android -----
+            // END Android-changed: App compat
         }
 
         if (host == null) {
@@ -267,7 +279,9 @@ public abstract class URLStreamHandler {
 
         // Parse the file path if any
         if (start < limit) {
-            if (spec.charAt(start) == '/') {
+            // Android-changed: Check for all hostname termination chars. http://b/110955991
+            // if (spec.charAt(start) == '/') {
+            if (spec.charAt(start) == '/' || spec.charAt(start) == '\\') {
                 path = spec.substring(start, limit);
             } else if (path != null && path.length() > 0) {
                 isRelPath = true;
@@ -283,21 +297,21 @@ public abstract class URLStreamHandler {
                 path = seperator + spec.substring(start, limit);
             }
         }
-        // ----- BEGIN android -----
+        // BEGIN Android-changed: App compat
         //else if (queryOnly && path != null) {
         //    int ind = path.lastIndexOf('/');
         //    if (ind < 0)
         //        ind = 0;
         //    path = path.substring(0, ind) + "/";
         //}
-        // ----- END android -----
+        // END Android-changed: App compat
         if (path == null)
             path = "";
 
-        // ----- BEGIN android -----
+        // BEGIN Android-changed
         //if (isRelPath) {
         if (true) {
-        // ----- END android -----
+        // END Android-changed
             // Remove embedded /./
             while ((i = path.indexOf("/./")) >= 0) {
                 path = path.substring(0, i) + path.substring(i + 2);
@@ -305,20 +319,22 @@ public abstract class URLStreamHandler {
             // Remove embedded /../ if possible
             i = 0;
             while ((i = path.indexOf("/../", i)) >= 0) {
-                // ----- BEGIN android -----
+                // BEGIN Android-changed: App compat
                 /*
                  * Trailing /../
                  */
                 if (i == 0) {
                     path = path.substring(i + 3);
                     i = 0;
-                // ----- END android -----
+                // END Android-changed: App compat
                 /*
                  * A "/../" will cancel the previous segment and itself,
                  * unless that segment is a "/../" itself
                  * i.e. "/a/b/../c" becomes "/a/c"
                  * but "/../../a" should stay unchanged
                  */
+                // Android-changed: App compat
+                // if (i > 0 && (limit = path.lastIndexOf('/', i - 1)) >= 0 &&
                 } else if (i > 0 && (limit = path.lastIndexOf('/', i - 1)) >= 0 &&
                     (path.indexOf("/../", limit) != 0)) {
                     path = path.substring(0, limit) + path.substring(i + 3);
@@ -344,7 +360,7 @@ public abstract class URLStreamHandler {
             if (path.endsWith("/."))
                 path = path.substring(0, path.length() -1);
 
-            // Remove trailing ?
+            // Android-changed: App compat: Remove trailing ?
             if (path.endsWith("?"))
                 path = path.substring(0, path.length() -1);
         }
@@ -355,7 +371,7 @@ public abstract class URLStreamHandler {
     /**
      * Returns the default port for a URL parsed by this handler. This method
      * is meant to be overidden by handlers with default port numbers.
-     * @return the default port for a <code>URL</code> parsed by this handler.
+     * @return the default port for a {@code URL} parsed by this handler.
      * @since 1.3
      */
     protected int getDefaultPort() {
@@ -369,12 +385,13 @@ public abstract class URLStreamHandler {
      * guaranteed by the fact that it is only called by java.net.URL class.
      * @param u1 a URL object
      * @param u2 a URL object
-     * @return <tt>true</tt> if the two urls are
+     * @return {@code true} if the two urls are
      * considered equal, ie. they refer to the same
      * fragment in the same file.
      * @since 1.3
      */
     protected boolean equals(URL u1, URL u2) {
+        // Android-changed: Avoid network I/O
         return Objects.equals(u1.getRef(), u2.getRef()) &&
                Objects.equals(u1.getQuery(), u2.getQuery()) &&
                // sameFile compares the protocol, file, port & host components of
@@ -387,10 +404,11 @@ public abstract class URLStreamHandler {
      * other protocols that have different requirements for hashCode
      * calculation.
      * @param u a URL object
-     * @return an <tt>int</tt> suitable for hash table indexing
+     * @return an {@code int} suitable for hash table indexing
      * @since 1.3
      */
     protected int hashCode(URL u) {
+        // Android-changed: Avoid network I/O
         // Hash on the same set of fields that we compare in equals().
         return Objects.hash(
                 u.getRef(),
@@ -443,7 +461,7 @@ public abstract class URLStreamHandler {
      * will result in a null return.
      *
      * @param u a URL object
-     * @return an <code>InetAddress</code> representing the host
+     * @return an {@code InetAddress} representing the host
      * IP address.
      * @since 1.3
      */
@@ -470,12 +488,12 @@ public abstract class URLStreamHandler {
      * Compares the host components of two URLs.
      * @param u1 the URL of the first host to compare
      * @param u2 the URL of the second host to compare
-     * @return  <tt>true</tt> if and only if they
-     * are equal, <tt>false</tt> otherwise.
+     * @return  {@code true} if and only if they
+     * are equal, {@code false} otherwise.
      * @since 1.3
      */
     protected boolean hostsEqual(URL u1, URL u2) {
-        // Android changed: Don't compare the InetAddresses of the hosts.
+        // Android-changed: Don't compare the InetAddresses of the hosts.
         if (u1.getHost() != null && u2.getHost() != null)
             return u1.getHost().equalsIgnoreCase(u2.getHost());
          else
@@ -483,13 +501,14 @@ public abstract class URLStreamHandler {
     }
 
     /**
-     * Converts a <code>URL</code> of a specific protocol to a
-     * <code>String</code>.
+     * Converts a {@code URL} of a specific protocol to a
+     * {@code String}.
      *
      * @param   u   the URL.
-     * @return  a string representation of the <code>URL</code> argument.
+     * @return  a string representation of the {@code URL} argument.
      */
     protected String toExternalForm(URL u) {
+
         // pre-compute length of StringBuffer
         int len = u.getProtocol().length() + 1;
         if (u.getAuthority() != null && u.getAuthority().length() > 0)
@@ -503,6 +522,8 @@ public abstract class URLStreamHandler {
         if (u.getRef() != null)
             len += 1 + u.getRef().length();
 
+        // BEGIN Android-changed: Add a toExternalForm variant that optionally escapes illegal chars
+        // TODO: The variant has been removed. We can potentially revert the change
         StringBuilder result = new StringBuilder(len);
         result.append(u.getProtocol());
         result.append(":");
@@ -514,6 +535,7 @@ public abstract class URLStreamHandler {
         if (fileAndQuery != null) {
             result.append(fileAndQuery);
         }
+        // END Android-changed: Add a toExternalForm variant that optionally escapes illegal chars
         if (u.getRef() != null) {
             result.append("#");
             result.append(u.getRef());
@@ -521,10 +543,12 @@ public abstract class URLStreamHandler {
         return result.toString();
     }
 
+    // Android-changed: Removed @see tag (target is package-private):
+    // @see     java.net.URL#set(java.lang.String, java.lang.String, int, java.lang.String, java.lang.String)
     /**
-     * Sets the fields of the <code>URL</code> argument to the indicated values.
-     * Only classes derived from URLStreamHandler are supposed to be able
-     * to call the set method on a URL.
+     * Sets the fields of the {@code URL} argument to the indicated values.
+     * Only classes derived from URLStreamHandler are able
+     * to use this method to set the values of the URL fields.
      *
      * @param   u         the URL to modify.
      * @param   protocol  the protocol name.
@@ -537,7 +561,6 @@ public abstract class URLStreamHandler {
      * @param   ref       the reference.
      * @exception       SecurityException       if the protocol handler of the URL is
      *                                  different from this one
-     * @see     java.net.URL#set(java.lang.String, java.lang.String, int, java.lang.String, java.lang.String)
      * @since 1.3
      */
        protected void setURL(URL u, String protocol, String host, int port,
@@ -552,9 +575,9 @@ public abstract class URLStreamHandler {
     }
 
     /**
-     * Sets the fields of the <code>URL</code> argument to the indicated values.
-     * Only classes derived from URLStreamHandler are supposed to be able
-     * to call the set method on a URL.
+     * Sets the fields of the {@code URL} argument to the indicated values.
+     * Only classes derived from URLStreamHandler are able
+     * to use this method to set the values of the URL fields.
      *
      * @param   u         the URL to modify.
      * @param   protocol  the protocol name. This value is ignored since 1.2.
