@@ -37,40 +37,40 @@ import java.util.Spliterator;
  *
  * <blockquote>
  *
- * <p> A buffer's <i>capacity</i> is the number of elements it contains.  The
- * capacity of a buffer is never negative and never changes.  </p>
+ *   <p> A buffer's <i>capacity</i> is the number of elements it contains.  The
+ *   capacity of a buffer is never negative and never changes.  </p>
  *
- * <p> A buffer's <i>limit</i> is the index of the first element that should
- * not be read or written.  A buffer's limit is never negative and is never
- * greater than its capacity.  </p>
+ *   <p> A buffer's <i>limit</i> is the index of the first element that should
+ *   not be read or written.  A buffer's limit is never negative and is never
+ *   greater than its capacity.  </p>
  *
- * <p> A buffer's <i>position</i> is the index of the next element to be
- * read or written.  A buffer's position is never negative and is never
- * greater than its limit.  </p>
+ *   <p> A buffer's <i>position</i> is the index of the next element to be
+ *   read or written.  A buffer's position is never negative and is never
+ *   greater than its limit.  </p>
  *
  * </blockquote>
  *
  * <p> There is one subclass of this class for each non-boolean primitive type.
  *
  *
- * <h4> Transferring data </h4>
+ * <h2> Transferring data </h2>
  *
  * <p> Each subclass of this class defines two categories of <i>get</i> and
  * <i>put</i> operations: </p>
  *
  * <blockquote>
  *
- * <p> <i>Relative</i> operations read or write one or more elements starting
- * at the current position and then increment the position by the number of
- * elements transferred.  If the requested transfer exceeds the limit then a
- * relative <i>get</i> operation throws a {@link BufferUnderflowException}
- * and a relative <i>put</i> operation throws a {@link
- * BufferOverflowException}; in either case, no data is transferred.  </p>
+ *   <p> <i>Relative</i> operations read or write one or more elements starting
+ *   at the current position and then increment the position by the number of
+ *   elements transferred.  If the requested transfer exceeds the limit then a
+ *   relative <i>get</i> operation throws a {@link BufferUnderflowException}
+ *   and a relative <i>put</i> operation throws a {@link
+ *   BufferOverflowException}; in either case, no data is transferred.  </p>
  *
- * <p> <i>Absolute</i> operations take an explicit element index and do not
- * affect the position.  Absolute <i>get</i> and <i>put</i> operations throw
- * an {@link IndexOutOfBoundsException} if the index argument exceeds the
- * limit.  </p>
+ *   <p> <i>Absolute</i> operations take an explicit element index and do not
+ *   affect the position.  Absolute <i>get</i> and <i>put</i> operations throw
+ *   an {@link IndexOutOfBoundsException} if the index argument exceeds the
+ *   limit.  </p>
  *
  * </blockquote>
  *
@@ -79,7 +79,7 @@ import java.util.Spliterator;
  * current position.
  *
  *
- * <h4> Marking and resetting </h4>
+ * <h2> Marking and resetting </h2>
  *
  * <p> A buffer's <i>mark</i> is the index to which its position will be reset
  * when the {@link #reset reset} method is invoked.  The mark is not always
@@ -90,17 +90,17 @@ import java.util.Spliterator;
  * {@link InvalidMarkException} to be thrown.
  *
  *
- * <h4> Invariants </h4>
+ * <h2> Invariants </h2>
  *
  * <p> The following invariant holds for the mark, position, limit, and
  * capacity values:
  *
  * <blockquote>
- * <tt>0</tt> <tt>&lt;=</tt>
- * <i>mark</i> <tt>&lt;=</tt>
- * <i>position</i> <tt>&lt;=</tt>
- * <i>limit</i> <tt>&lt;=</tt>
- * <i>capacity</i>
+ *     <tt>0</tt> <tt>&lt;=</tt>
+ *     <i>mark</i> <tt>&lt;=</tt>
+ *     <i>position</i> <tt>&lt;=</tt>
+ *     <i>limit</i> <tt>&lt;=</tt>
+ *     <i>capacity</i>
  * </blockquote>
  *
  * <p> A newly-created buffer always has a position of zero and a mark that is
@@ -110,7 +110,7 @@ import java.util.Spliterator;
  * to zero.
  *
  *
- * <h4> Clearing, flipping, and rewinding </h4>
+ * <h2> Clearing, flipping, and rewinding </h2>
  *
  * <p> In addition to methods for accessing the position, limit, and capacity
  * values and for marking and resetting, this class also defines the following
@@ -118,22 +118,22 @@ import java.util.Spliterator;
  *
  * <ul>
  *
- * <li><p> {@link #clear} makes a buffer ready for a new sequence of
- * channel-read or relative <i>put</i> operations: It sets the limit to the
- * capacity and the position to zero.  </p></li>
+ *   <li><p> {@link #clear} makes a buffer ready for a new sequence of
+ *   channel-read or relative <i>put</i> operations: It sets the limit to the
+ *   capacity and the position to zero.  </p></li>
  *
- * <li><p> {@link #flip} makes a buffer ready for a new sequence of
- * channel-write or relative <i>get</i> operations: It sets the limit to the
- * current position and then sets the position to zero.  </p></li>
+ *   <li><p> {@link #flip} makes a buffer ready for a new sequence of
+ *   channel-write or relative <i>get</i> operations: It sets the limit to the
+ *   current position and then sets the position to zero.  </p></li>
  *
- * <li><p> {@link #rewind} makes a buffer ready for re-reading the data that
- * it already contains: It leaves the limit unchanged and sets the position
- * to zero.  </p></li>
+ *   <li><p> {@link #rewind} makes a buffer ready for re-reading the data that
+ *   it already contains: It leaves the limit unchanged and sets the position
+ *   to zero.  </p></li>
  *
  * </ul>
  *
  *
- * <h4> Read-only buffers </h4>
+ * <h2> Read-only buffers </h2>
  *
  * <p> Every buffer is readable, but not every buffer is writable.  The
  * mutation methods of each buffer class are specified as <i>optional
@@ -144,14 +144,14 @@ import java.util.Spliterator;
  * {@link #isReadOnly isReadOnly} method.
  *
  *
- * <h4> Thread safety </h4>
+ * <h2> Thread safety </h2>
  *
  * <p> Buffers are not safe for use by multiple concurrent threads.  If a
  * buffer is to be used by more than one thread then access to the buffer
  * should be controlled by appropriate synchronization.
  *
  *
- * <h4> Invocation chaining </h4>
+ * <h2> Invocation chaining </h2>
  *
  * <p> Methods in this class that do not otherwise have a value to return are
  * specified to return the buffer upon which they are invoked.  This allows
@@ -167,6 +167,7 @@ import java.util.Spliterator;
  * <blockquote><pre>
  * b.flip().position(23).limit(42);</pre></blockquote>
  *
+ *
  * @author Mark Reinhold
  * @author JSR-51 Expert Group
  * @since 1.4
@@ -183,6 +184,7 @@ public abstract class Buffer {
 
     // Invariants: mark <= position <= limit <= capacity
     private int mark = -1;
+    // Android-changed: position field non-private for use by Android's nio implementation classes.
     int position = 0;
     private int limit;
     private int capacity;
@@ -191,6 +193,7 @@ public abstract class Buffer {
     // NOTE: hoisted here for speed in JNI GetDirectBufferAddress
     long address;
 
+    // Android-added: _elementSizeShift field for NIOAccess class and framework native code.
     /**
      * The log base 2 of the element size of this buffer.  Each typed subclass
      * (ByteBuffer, CharBuffer, etc.) is responsible for initializing this
@@ -202,6 +205,7 @@ public abstract class Buffer {
     // Creates a new buffer with the given mark, position, limit, and capacity,
     // after checking invariants.
     //
+    // Android-added: _elementSizeShift field for NIOAccess class and framework native code.
     Buffer(int mark, int pos, int lim, int cap, int elementSizeShift) {       // package-private
         if (cap < 0)
             throw new IllegalArgumentException("Negative capacity: " + cap);
@@ -211,25 +215,26 @@ public abstract class Buffer {
         if (mark >= 0) {
             if (mark > pos)
                 throw new IllegalArgumentException("mark > position: ("
-                        + mark + " > " + pos + ")");
+                                                   + mark + " > " + pos + ")");
             this.mark = mark;
         }
+        // Android-added: _elementSizeShift field for NIOAccess class and framework native code.
         _elementSizeShift = elementSizeShift;
     }
 
     /**
-     * Returns this buffer's capacity. </p>
+     * Returns this buffer's capacity.
      *
-     * @return The capacity of this buffer
+     * @return  The capacity of this buffer
      */
     public final int capacity() {
         return capacity;
     }
 
     /**
-     * Returns this buffer's position. </p>
+     * Returns this buffer's position.
      *
-     * @return The position of this buffer
+     * @return  The position of this buffer
      */
     public final int position() {
         return position;
@@ -237,15 +242,20 @@ public abstract class Buffer {
 
     /**
      * Sets this buffer's position.  If the mark is defined and larger than the
-     * new position then it is discarded. </p>
+     * new position then it is discarded.
      *
-     * @param newPosition The new position value; must be non-negative
-     *                    and no larger than the current limit
-     * @return This buffer
-     * @throws IllegalArgumentException If the preconditions on <tt>newPosition</tt> do not hold
+     * @param  newPosition
+     *         The new position value; must be non-negative
+     *         and no larger than the current limit
+     *
+     * @return  This buffer
+     *
+     * @throws  IllegalArgumentException
+     *          If the preconditions on <tt>newPosition</tt> do not hold
      */
-    public final Buffer position(int newPosition) {
+    public Buffer position(int newPosition) {
         if ((newPosition > limit) || (newPosition < 0))
+            // Android-changed: Improved error message.
             throw new IllegalArgumentException("Bad position " + newPosition + "/" + limit);
         position = newPosition;
         if (mark > position) mark = -1;
@@ -253,9 +263,9 @@ public abstract class Buffer {
     }
 
     /**
-     * Returns this buffer's limit. </p>
+     * Returns this buffer's limit.
      *
-     * @return The limit of this buffer
+     * @return  The limit of this buffer
      */
     public final int limit() {
         return limit;
@@ -264,14 +274,18 @@ public abstract class Buffer {
     /**
      * Sets this buffer's limit.  If the position is larger than the new limit
      * then it is set to the new limit.  If the mark is defined and larger than
-     * the new limit then it is discarded. </p>
+     * the new limit then it is discarded.
      *
-     * @param newLimit The new limit value; must be non-negative
-     *                 and no larger than this buffer's capacity
-     * @return This buffer
-     * @throws IllegalArgumentException If the preconditions on <tt>newLimit</tt> do not hold
+     * @param  newLimit
+     *         The new limit value; must be non-negative
+     *         and no larger than this buffer's capacity
+     *
+     * @return  This buffer
+     *
+     * @throws  IllegalArgumentException
+     *          If the preconditions on <tt>newLimit</tt> do not hold
      */
-    public final Buffer limit(int newLimit) {
+    public Buffer limit(int newLimit) {
         if ((newLimit > capacity) || (newLimit < 0))
             throw new IllegalArgumentException();
         limit = newLimit;
@@ -281,11 +295,11 @@ public abstract class Buffer {
     }
 
     /**
-     * Sets this buffer's mark at its position. </p>
+     * Sets this buffer's mark at its position.
      *
-     * @return This buffer
+     * @return  This buffer
      */
-    public final Buffer mark() {
+    public Buffer mark() {
         mark = position;
         return this;
     }
@@ -296,10 +310,12 @@ public abstract class Buffer {
      * <p> Invoking this method neither changes nor discards the mark's
      * value. </p>
      *
-     * @return This buffer
-     * @throws InvalidMarkException If the mark has not been set
+     * @return  This buffer
+     *
+     * @throws  InvalidMarkException
+     *          If the mark has not been set
      */
-    public final Buffer reset() {
+    public Buffer reset() {
         int m = mark;
         if (m < 0)
             throw new InvalidMarkException();
@@ -322,9 +338,9 @@ public abstract class Buffer {
      * is named as if it did because it will most often be used in situations
      * in which that might as well be the case. </p>
      *
-     * @return This buffer
+     * @return  This buffer
      */
-    public final Buffer clear() {
+    public Buffer clear() {
         position = 0;
         limit = capacity;
         mark = -1;
@@ -350,9 +366,9 @@ public abstract class Buffer {
      * java.nio.ByteBuffer#compact compact} method when transferring data from
      * one place to another.  </p>
      *
-     * @return This buffer
+     * @return  This buffer
      */
-    public final Buffer flip() {
+    public Buffer flip() {
         limit = position;
         position = 0;
         mark = -1;
@@ -372,9 +388,9 @@ public abstract class Buffer {
      * buf.rewind();      // Rewind buffer
      * buf.get(array);    // Copy data into array</pre></blockquote>
      *
-     * @return This buffer
+     * @return  This buffer
      */
-    public final Buffer rewind() {
+    public Buffer rewind() {
         position = 0;
         mark = -1;
         return this;
@@ -382,9 +398,9 @@ public abstract class Buffer {
 
     /**
      * Returns the number of elements between the current position and the
-     * limit. </p>
+     * limit.
      *
-     * @return The number of elements remaining in this buffer
+     * @return  The number of elements remaining in this buffer
      */
     public final int remaining() {
         return limit - position;
@@ -392,19 +408,19 @@ public abstract class Buffer {
 
     /**
      * Tells whether there are any elements between the current position and
-     * the limit. </p>
+     * the limit.
      *
-     * @return <tt>true</tt> if, and only if, there is at least one element
-     * remaining in this buffer
+     * @return  <tt>true</tt> if, and only if, there is at least one element
+     *          remaining in this buffer
      */
     public final boolean hasRemaining() {
         return position < limit;
     }
 
     /**
-     * Tells whether or not this buffer is read-only. </p>
+     * Tells whether or not this buffer is read-only.
      *
-     * @return <tt>true</tt> if, and only if, this buffer is read-only
+     * @return  <tt>true</tt> if, and only if, this buffer is read-only
      */
     public abstract boolean isReadOnly();
 
@@ -416,8 +432,9 @@ public abstract class Buffer {
      * and {@link #arrayOffset() arrayOffset} methods may safely be invoked.
      * </p>
      *
-     * @return <tt>true</tt> if, and only if, this buffer
-     * is backed by an array and is not read-only
+     * @return  <tt>true</tt> if, and only if, this buffer
+     *          is backed by an array and is not read-only
+     *
      * @since 1.6
      */
     public abstract boolean hasArray();
@@ -437,9 +454,14 @@ public abstract class Buffer {
      * method in order to ensure that this buffer has an accessible backing
      * array.  </p>
      *
-     * @return The array that backs this buffer
-     * @throws ReadOnlyBufferException       If this buffer is backed by an array but is read-only
-     * @throws UnsupportedOperationException If this buffer is not backed by an accessible array
+     * @return  The array that backs this buffer
+     *
+     * @throws  ReadOnlyBufferException
+     *          If this buffer is backed by an array but is read-only
+     *
+     * @throws  UnsupportedOperationException
+     *          If this buffer is not backed by an accessible array
+     *
      * @since 1.6
      */
     public abstract Object array();
@@ -455,19 +477,25 @@ public abstract class Buffer {
      * method in order to ensure that this buffer has an accessible backing
      * array.  </p>
      *
-     * @return The offset within this buffer's array
-     * of the first element of the buffer
-     * @throws ReadOnlyBufferException       If this buffer is backed by an array but is read-only
-     * @throws UnsupportedOperationException If this buffer is not backed by an accessible array
+     * @return  The offset within this buffer's array
+     *          of the first element of the buffer
+     *
+     * @throws  ReadOnlyBufferException
+     *          If this buffer is backed by an array but is read-only
+     *
+     * @throws  UnsupportedOperationException
+     *          If this buffer is not backed by an accessible array
+     *
      * @since 1.6
      */
     public abstract int arrayOffset();
 
     /**
      * Tells whether or not this buffer is
-     * <a href="ByteBuffer.html#direct"><i>direct</i></a>. </p>
+     * <a href="ByteBuffer.html#direct"><i>direct</i></a>.
+     *
+     * @return  <tt>true</tt> if, and only if, this buffer is direct
      *
-     * @return <tt>true</tt> if, and only if, this buffer is direct
      * @since 1.6
      */
     public abstract boolean isDirect();
@@ -478,9 +506,9 @@ public abstract class Buffer {
     /**
      * Checks the current position against the limit, throwing a {@link
      * BufferUnderflowException} if it is not smaller than the limit, and then
-     * increments the position. </p>
+     * increments the position.
      *
-     * @return The current position value, before it is incremented
+     * @return  The current position value, before it is incremented
      */
     final int nextGetIndex() {                          // package-private
         if (position >= limit)
@@ -499,9 +527,9 @@ public abstract class Buffer {
     /**
      * Checks the current position against the limit, throwing a {@link
      * BufferOverflowException} if it is not smaller than the limit, and then
-     * increments the position. </p>
+     * increments the position.
      *
-     * @return The current position value, before it is incremented
+     * @return  The current position value, before it is incremented
      */
     final int nextPutIndex() {                          // package-private
         if (position >= limit)
@@ -524,7 +552,7 @@ public abstract class Buffer {
      */
     final int checkIndex(int i) {                       // package-private
         if ((i < 0) || (i >= limit))
-            // Android changed : Add bounds details to exception.
+            // Android-changed: Add bounds details to exception.
             throw new IndexOutOfBoundsException(
                     "index=" + i + " out of bounds (limit=" + limit + ")");
         return i;
@@ -532,7 +560,7 @@ public abstract class Buffer {
 
     final int checkIndex(int i, int nb) {               // package-private
         if ((i < 0) || (nb > limit - i))
-            // Android changed : Add bounds details to exception.
+            // Android-changed: Add bounds details to exception.
             throw new IndexOutOfBoundsException(
                     "index=" + i + " out of bounds (limit=" + limit + ", nb=" + nb + ")");
         return i;
@@ -555,11 +583,12 @@ public abstract class Buffer {
 
     static void checkBounds(int off, int len, int size) { // package-private
         if ((off | len | (off + len) | (size - (off + len))) < 0)
-            // Android changed : Add bounds details to exception.
+            // Android-changed: Add bounds details to exception.
             throw new IndexOutOfBoundsException(
                     "off=" + off + ", len=" + len + " out of bounds (size=" + size + ")");
     }
 
+    // Android-added: getElementSizeShift() method for testing.
     /**
      * For testing only. This field is accessed directly via JNI from frameworks code.
      *
