@@ -25,7 +25,9 @@
 
 package sun.misc;
 
-import dalvik.system.VMStack;
+import dalvik.annotation.optimization.FastNative;
+import sun.reflect.Reflection;
+
 import java.lang.reflect.Field;
 import java.lang.reflect.Modifier;
 
@@ -54,11 +56,12 @@ public final class Unsafe {
      * very limited situations.
      */
     public static Unsafe getUnsafe() {
+        Class<?> caller = Reflection.getCallerClass();
         /*
          * Only code on the bootclasspath is allowed to get at the
          * Unsafe instance.
          */
-        ClassLoader calling = VMStack.getCallingClassLoader();
+        ClassLoader calling = (caller == null) ? null : caller.getClassLoader();
         if ((calling != null) && (calling != Unsafe.class.getClassLoader())) {
             throw new SecurityException("Unsafe access denied");
         }
@@ -110,7 +113,9 @@ public final class Unsafe {
       return getArrayIndexScaleForComponentType(component);
     }
 
+    @FastNative
     private static native int getArrayBaseOffsetForComponentType(Class component_class);
+    @FastNative
     private static native int getArrayIndexScaleForComponentType(Class component_class);
 
     /**
@@ -125,6 +130,7 @@ public final class Unsafe {
      * @return <code>true</code> if the new value was in fact stored, and
      * <code>false</code> if not
      */
+    @FastNative
     public native boolean compareAndSwapInt(Object obj, long offset,
             int expectedValue, int newValue);
 
@@ -140,6 +146,7 @@ public final class Unsafe {
      * @return <code>true</code> if the new value was in fact stored, and
      * <code>false</code> if not
      */
+    @FastNative
     public native boolean compareAndSwapLong(Object obj, long offset,
             long expectedValue, long newValue);
 
@@ -155,6 +162,7 @@ public final class Unsafe {
      * @return <code>true</code> if the new value was in fact stored, and
      * <code>false</code> if not
      */
+    @FastNative
     public native boolean compareAndSwapObject(Object obj, long offset,
             Object expectedValue, Object newValue);
 
@@ -166,6 +174,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @return the retrieved value
      */
+    @FastNative
     public native int getIntVolatile(Object obj, long offset);
 
     /**
@@ -176,6 +185,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @param newValue the value to store
      */
+    @FastNative
     public native void putIntVolatile(Object obj, long offset, int newValue);
 
     /**
@@ -186,6 +196,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @return the retrieved value
      */
+    @FastNative
     public native long getLongVolatile(Object obj, long offset);
 
     /**
@@ -196,6 +207,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @param newValue the value to store
      */
+    @FastNative
     public native void putLongVolatile(Object obj, long offset, long newValue);
 
     /**
@@ -206,6 +218,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @return the retrieved value
      */
+    @FastNative
     public native Object getObjectVolatile(Object obj, long offset);
 
     /**
@@ -216,6 +229,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @param newValue the value to store
      */
+    @FastNative
     public native void putObjectVolatile(Object obj, long offset,
             Object newValue);
 
@@ -226,6 +240,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @return the retrieved value
      */
+    @FastNative
     public native int getInt(Object obj, long offset);
 
     /**
@@ -235,11 +250,13 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @param newValue the value to store
      */
+    @FastNative
     public native void putInt(Object obj, long offset, int newValue);
 
     /**
      * Lazy set an int field.
      */
+    @FastNative
     public native void putOrderedInt(Object obj, long offset, int newValue);
 
     /**
@@ -249,6 +266,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @return the retrieved value
      */
+    @FastNative
     public native long getLong(Object obj, long offset);
 
     /**
@@ -258,11 +276,13 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @param newValue the value to store
      */
+    @FastNative
     public native void putLong(Object obj, long offset, long newValue);
 
     /**
      * Lazy set a long field.
      */
+    @FastNative
     public native void putOrderedLong(Object obj, long offset, long newValue);
 
     /**
@@ -272,6 +292,7 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @return the retrieved value
      */
+    @FastNative
     public native Object getObject(Object obj, long offset);
 
     /**
@@ -281,26 +302,40 @@ public final class Unsafe {
      * @param offset offset to the field within <code>obj</code>
      * @param newValue the value to store
      */
+    @FastNative
     public native void putObject(Object obj, long offset, Object newValue);
 
     /**
      * Lazy set an object field.
      */
+    @FastNative
     public native void putOrderedObject(Object obj, long offset,
             Object newValue);
 
 
+    @FastNative
     public native boolean getBoolean(Object obj, long offset);
+    @FastNative
     public native void putBoolean(Object obj, long offset, boolean newValue);
+    @FastNative
     public native byte getByte(Object obj, long offset);
+    @FastNative
     public native void putByte(Object obj, long offset, byte newValue);
+    @FastNative
     public native char getChar(Object obj, long offset);
+    @FastNative
     public native void putChar(Object obj, long offset, char newValue);
+    @FastNative
     public native short getShort(Object obj, long offset);
+    @FastNative
     public native void putShort(Object obj, long offset, short newValue);
+    @FastNative
     public native float getFloat(Object obj, long offset);
+    @FastNative
     public native void putFloat(Object obj, long offset, float newValue);
+    @FastNative
     public native double getDouble(Object obj, long offset);
+    @FastNative
     public native void putDouble(Object obj, long offset, double newValue);
 
     /**
@@ -318,14 +353,7 @@ public final class Unsafe {
      * nanoseconds-from-now (<code>false</code>)
      * @param time the (absolute millis or relative nanos) time value
      */
-    public void park(boolean absolute, long time) {
-        if (absolute) {
-            Thread.currentThread().parkUntil$(time);
-        } else {
-            Thread.currentThread().parkFor$(time);
-        }
-    }
-
+    public native void park(boolean absolute, long time);
     /**
      * Unparks the given object, which must be a {@link Thread}.
      *
@@ -334,64 +362,80 @@ public final class Unsafe {
      *
      * @param obj non-null; the object to unpark
      */
-    public void unpark(Object obj) {
-        if (obj instanceof Thread) {
-            ((Thread) obj).unpark$();
-        } else {
-            throw new IllegalArgumentException("valid for Threads only");
-        }
-    }
-
+    @FastNative
+    public native void unpark(Object obj);
     /**
      * Allocates an instance of the given class without running the constructor.
      * The class' <clinit> will be run, if necessary.
      */
     public native Object allocateInstance(Class<?> c);
 
+    @FastNative
     public native int addressSize();
 
+    @FastNative
     public native int pageSize();
 
+    @FastNative
     public native long allocateMemory(long bytes);
 
+    @FastNative
     public native void freeMemory(long address);
 
+    @FastNative
     public native void setMemory(long address, long bytes, byte value);
 
+    @FastNative
     public native byte getByte(long address);
 
+    @FastNative
     public native void putByte(long address, byte x);
 
+    @FastNative
     public native short getShort(long address);
 
+    @FastNative
     public native void putShort(long address, short x);
 
+    @FastNative
     public native char getChar(long address);
 
+    @FastNative
     public native void putChar(long address, char x);
 
+    @FastNative
     public native int getInt(long address);
 
+    @FastNative
     public native void putInt(long address, int x);
 
+    @FastNative
     public native long getLong(long address);
 
+    @FastNative
     public native void putLong(long address, long x);
 
+    @FastNative
     public native float getFloat(long address);
 
+    @FastNative
     public native void putFloat(long address, float x);
 
+    @FastNative
     public native double getDouble(long address);
 
+    @FastNative
     public native void putDouble(long address, double x);
 
+    @FastNative
     public native void copyMemoryToPrimitiveArray(long srcAddr,
             Object dst, long dstOffset, long bytes);
 
+    @FastNative
     public native void copyMemoryFromPrimitiveArray(Object src, long srcOffset,
             long dstAddr, long bytes);
 
+    @FastNative
     public native void copyMemory(long srcAddr, long dstAddr, long bytes);
 
 
@@ -512,6 +556,7 @@ public final class Unsafe {
      * @since 1.8
      */
     // @HotSpotIntrinsicCandidate
+    @FastNative
     public native void loadFence();
 
     /**
@@ -527,6 +572,7 @@ public final class Unsafe {
      * @since 1.8
      */
     // @HotSpotIntrinsicCandidate
+    @FastNative
     public native void storeFence();
 
     /**
@@ -539,5 +585,6 @@ public final class Unsafe {
      * @since 1.8
      */
     // @HotSpotIntrinsicCandidate
+    @FastNative
     public native void fullFence();
 }
