@@ -16,7 +16,7 @@
 
 package libcore.java.security.cert;
 
-import tests.support.resource.Support_Resources;
+import static java.nio.charset.StandardCharsets.UTF_8;
 
 import java.io.BufferedInputStream;
 import java.io.BufferedReader;
@@ -56,13 +56,35 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.Set;
-
 import javax.security.auth.x500.X500Principal;
-
 import junit.framework.TestCase;
 import libcore.java.security.StandardNames;
 
+import dalvik.system.VMRuntime;
+import sun.security.jca.Providers;
+import tests.support.resource.Support_Resources;
+
 public class X509CertificateTest extends TestCase {
+
+    @Override
+    protected void setUp() throws Exception {
+        super.setUp();
+
+        mX509Providers = Security.getProviders("CertificateFactory.X509");
+
+        // Allow access to deprecated BC algorithms in this test, so we can ensure they
+        // continue to work
+        Providers.setMaximumAllowableApiLevelForBcDeprecation(
+                VMRuntime.getRuntime().getTargetSdkVersion());
+    }
+
+    @Override
+    public void tearDown() throws Exception {
+        Providers.setMaximumAllowableApiLevelForBcDeprecation(
+                Providers.DEFAULT_MAXIMUM_ALLOWABLE_TARGET_API_LEVEL_FOR_BC_DEPRECATION);
+        super.tearDown();
+    }
+
     private Provider[] mX509Providers;
 
     private static final String CERT_RSA = "x509/cert-rsa.der";
@@ -173,7 +195,7 @@ public class X509CertificateTest extends TestCase {
             final SimpleDateFormat sdf =
                     new SimpleDateFormat("MMM dd HH:mm:ss yyyy zzz", Locale.US);
 
-            final BufferedReader buf = new BufferedReader(new InputStreamReader(ris));
+            final BufferedReader buf = new BufferedReader(new InputStreamReader(ris, UTF_8));
             String line = buf.readLine();
             int index = line.indexOf('=');
             assertEquals("notBefore", line.substring(0, index));
@@ -199,7 +221,7 @@ public class X509CertificateTest extends TestCase {
     private BigInteger getRsaCertificateSerial() throws Exception {
         final InputStream ris = Support_Resources.getStream("x509/cert-rsa-serial.txt");
         try {
-            final BufferedReader buf = new BufferedReader(new InputStreamReader(ris));
+            final BufferedReader buf = new BufferedReader(new InputStreamReader(ris, UTF_8));
 
             String line = buf.readLine();
             int index = line.indexOf('=');
@@ -1141,12 +1163,7 @@ public class X509CertificateTest extends TestCase {
         /* PEM-encoded PKCS7 bag of certificates */
         Collection<? extends X509Certificate> certs = getCertificates(f, CERTS_PKCS7_PEM);
         assertNotNull(certs);
-        if ("BC".equals(f.getProvider().getName())) {
-            // Bouncycastle is broken
-            assertEquals(0, certs.size());
-        } else {
-            assertEquals(2, certs.size());
-        }
+        assertEquals(2, certs.size());
     }
 
     private void generateCertificates_PKCS7_DER(CertificateFactory f) throws Exception {
@@ -1235,12 +1252,7 @@ public class X509CertificateTest extends TestCase {
         Collection<? extends X509Certificate> certs = (Collection<? extends X509Certificate>)
                 f.generateCertificates(bais);
 
-        // Bouncycastle is broken
-        if ("BC".equals(f.getProvider().getName())) {
-            assertEquals(0, bais.available());
-        } else {
-            assertEquals(4096, bais.available());
-        }
+        assertEquals(4096, bais.available());
     }
 
     private void generateCertificates_PKCS7_DER_TrailingData(CertificateFactory f) throws Exception {
@@ -1338,11 +1350,4 @@ public class X509CertificateTest extends TestCase {
         }
         return certBytes;
     }
-
-    @Override
-    protected void setUp() throws Exception {
-        super.setUp();
-
-        mX509Providers = Security.getProviders("CertificateFactory.X509");
-    }
 }
