@@ -26,6 +26,7 @@ import java.security.Principal;
 import java.security.cert.Certificate;
 import java.security.cert.X509Certificate;
 import java.util.Arrays;
+import java.util.Base64;
 
 import javax.net.ssl.ExtendedSSLSession;
 import javax.net.ssl.KeyManager;
@@ -42,7 +43,6 @@ import org.apache.harmony.tests.javax.net.ssl.HandshakeCompletedEventTest.MyHand
 import org.apache.harmony.tests.javax.net.ssl.HandshakeCompletedEventTest.TestTrustManager;
 
 import junit.framework.TestCase;
-import libcore.io.Base64;
 import libcore.java.security.StandardNames;
 
 public class SSLSessionTest extends TestCase {
@@ -90,27 +90,6 @@ public class SSLSessionTest extends TestCase {
         assertTrue(clientSession.getApplicationBufferSize() > 0);
     }
 
-    /**
-     * javax.net.ssl.SSLSession#getCipherSuite()
-     */
-    public void test_getCipherSuite() {
-        // Identify the expected cipher suite from the expected list of cipher suites enabled by
-        // default.
-        // This test class initializes the server with an RSA key. Thus, only cipher suites that
-        // authenticate the server using RSA are expected to be used.
-        String expectedCipherSuite = null;
-        for (String cipherSuite : StandardNames.CIPHER_SUITES_DEFAULT) {
-            if (cipherSuite.contains("_RSA_")) {
-                expectedCipherSuite = cipherSuite;
-                break;
-            }
-        }
-        if (expectedCipherSuite == null) {
-            fail("Failed to identify expected cipher suite");
-        }
-        assertEquals(expectedCipherSuite, clientSession.getCipherSuite());
-    }
-
     /**
      * javax.net.ssl.SSLSession#getCreationTime()
      */
@@ -123,22 +102,6 @@ public class SSLSessionTest extends TestCase {
                    diff < 10000);
     }
 
-    /**
-     * javax.net.ssl.SSLSession#getId()
-     */
-    public void test_getId() throws Exception {
-        byte[] id = clientSession.getId();
-        SSLSession sess = clientSslContext.getClientSessionContext().getSession(id);
-        assertNotNull("Could not find session for id " + id, sess);
-        if (clientSession instanceof ExtendedSSLSession) {
-            assertTrue(sess instanceof ExtendedSSLSession);
-            assertExtendedSSLSessionsEqual((ExtendedSSLSession) clientSession,
-                    (ExtendedSSLSession) sess);
-        } else {
-            assertSSLSessionsEqual(clientSession, sess);
-        }
-    }
-
     /**
      * javax.net.ssl.SSLSession#getLastAccessedTime()
      */
@@ -200,7 +163,7 @@ public class SSLSessionTest extends TestCase {
      * javax.net.ssl.SSLSession#getProtocol()
      */
     public void test_getProtocol() {
-        assertEquals("TLSv1.2", clientSession.getProtocol());
+        assertEquals("TLSv1.3", clientSession.getProtocol());
     }
 
     /**
@@ -230,27 +193,27 @@ public class SSLSessionTest extends TestCase {
 
         try {
             s.putValue(null, null);
-            fail("IllegalArgumentException wasn't thrown");
-        } catch (IllegalArgumentException expected) {
+            fail("Exception wasn't thrown");
+        } catch (IllegalArgumentException | NullPointerException expected) {
             // expected
         }
         try {
             s.putValue("ABC", null);
-            fail("IllegalArgumentException wasn't thrown");
-        } catch (IllegalArgumentException expected) {
+            fail("Exception wasn't thrown");
+        } catch (IllegalArgumentException | NullPointerException expected) {
             // expected
         }
         try {
             s.putValue(null, sbl);
-            fail("IllegalArgumentException wasn't thrown");
-        } catch (IllegalArgumentException expected) {
+            fail("Exception wasn't thrown");
+        } catch (IllegalArgumentException | NullPointerException expected) {
             // expected
         }
 
         try {
             s.removeValue(null);
-            fail("IllegalArgumentException wasn't thrown");
-        } catch (IllegalArgumentException expected) {
+            fail("Exception wasn't thrown");
+        } catch (IllegalArgumentException | NullPointerException expected) {
             // expected
         }
     }
@@ -264,8 +227,8 @@ public class SSLSessionTest extends TestCase {
 
         try {
             s.getValue(null);
-            fail("IllegalArgumentException wasn't thrown");
-        } catch (IllegalArgumentException expected) {
+            fail("Exception wasn't thrown");
+        } catch (IllegalArgumentException | NullPointerException expected) {
             // expected
         }
 
@@ -643,7 +606,7 @@ public class SSLSessionTest extends TestCase {
      * for the result.
      */
     private KeyStore getKeyStore(String keys) throws Exception {
-        byte[] bytes = Base64.decode(keys.getBytes());
+        byte[] bytes = Base64.getDecoder().decode(keys.getBytes());
         InputStream inputStream = new ByteArrayInputStream(bytes);
 
         KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
