@@ -30,12 +30,32 @@ import java.security.cert.PKIXCertPathValidatorResult;
 import java.security.cert.PKIXParameters;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
+import java.util.Calendar;
 import java.util.List;
 import junit.framework.TestCase;
 import libcore.java.security.StandardNames;
 
+import dalvik.system.VMRuntime;
+import sun.security.jca.Providers;
+
 public class CertificateTest extends TestCase {
 
+    // Allow access to deprecated BC algorithms in this test, so we can ensure they
+    // continue to work
+    @Override
+    public void setUp() throws Exception {
+        super.setUp();
+        Providers.setMaximumAllowableApiLevelForBcDeprecation(
+                VMRuntime.getRuntime().getTargetSdkVersion());
+    }
+
+    @Override
+    public void tearDown() throws Exception {
+        Providers.setMaximumAllowableApiLevelForBcDeprecation(
+                Providers.DEFAULT_MAXIMUM_ALLOWABLE_TARGET_API_LEVEL_FOR_BC_DEPRECATION);
+        super.tearDown();
+    }
+
     /*
      * Following certificate chain was taken from https://www.verisign.com and
      * uses MD2withRSA for the root certificate. This chain stops validating
@@ -737,6 +757,15 @@ public class CertificateTest extends TestCase {
         PKIXParameters params;
         params = new PKIXParameters(keyStore);
         params.setRevocationEnabled(false);
+
+        // All the tests are using pre-generated certificates with set expirations.
+        // In order to avoid the test failing when the certificates expire,
+        // explicitly set the time to check their validity against.
+        Calendar calendar = Calendar.getInstance();
+        calendar.clear();
+        calendar.set(2016, Calendar.JANUARY, 1);
+        params.setDate(calendar.getTime());
+
         return params;
     }
 }
